@@ -4,10 +4,11 @@ use strict;
 use warnings;
 use Exporter 'import';
 
-our $VERSION = "0.03";
+our $VERSION = "0.04";
 
 our @EXPORT = qw(
     compress
+    compress_mt
     decompress
     uncompress
     ZSTD_VERSION_NUMBER
@@ -47,6 +48,12 @@ buffer can be either a scalar or a scalar reference.
 
 On error undef is returned.
 
+=head2 compress_mt($source, $num_threads [, $level])
+
+Multi-threaded version of the C<compress> function.
+
+Note that this function uses experimental API of Zstandard.
+
 =head2 decompress($source)
 
 =head2 uncompress($source)
